"""Buy bot"""

import re
import os

from bs4 import BeautifulSoup
from rival_regions_wrapper.middleware import LocalAuthentication
from rival_regions_wrapper.wrapper.abstract_wrapper import AbstractWrapper
from rival_regions_wrapper import util


class MissingAuthenticationError(Exception):
    """Error for missing authentication"""


RIVAL_REGIONS_USERNAME = os.environ.get("RIVAL_REGIONS_USERNAME", None)
RIVAL_REGIONS_PASSWORD = os.environ.get("RIVAL_REGIONS_PASSWORD", None)
RIVAL_REGIONS_LOGIN_METHOD = os.environ.get("RIVAL_REGIONS_LOGIN_METHOD", None)
CAPTCHA_KEY = os.environ.get("CAPTCHA_KEY", None)

if None in (
    RIVAL_REGIONS_USERNAME,
    RIVAL_REGIONS_PASSWORD,
    RIVAL_REGIONS_LOGIN_METHOD,
):
    raise MissingAuthenticationError(
        "Load the following variables in your user environment: "
        "RIVAL_REGIONS_USERNAME, RIVAL_REGIONS_PASSWORD, RIVAL_REGIONS_LOGIN_METHOD"
    )

authentication = LocalAuthentication(False, CAPTCHA_KEY)
authentication.set_credentials(
    RIVAL_REGIONS_USERNAME, RIVAL_REGIONS_PASSWORD, RIVAL_REGIONS_LOGIN_METHOD
)


class CurrentPrice(AbstractWrapper):
    def info(self, resource):
        """Get profile"""
        if isinstance(resource, str) and resource in util.ITEM_KEYS:
            resource = util.ITEM_KEYS[resource]
        path = "storage/listed/{}".format(resource)
        response = self.middleware.get(path)
        soup = BeautifulSoup(response, "html.parser")
        offers_tree = soup.find_all(class_="list_link")
        test = 0
        prijs = 0
        for offer_tree in offers_tree:
            if test == 0:
                test = 1
                prijs = int(
                    float(offer_tree.select(".list_level")[1]["rat"]) * 100
                )
        return prijs


class Amount(AbstractWrapper):
    def info(self, resource):
        """Get profile"""
        if isinstance(resource, str) and resource in util.ITEM_KEYS:
            resource = util.ITEM_KEYS[resource]
        path = "storage/listed/{}".format(resource)
        response = self.middleware.get(path)
        soup = BeautifulSoup(response, "html.parser")
        offers_tree = soup.find_all(class_="list_link")
        test = 0
        for offer_tree in offers_tree:
            if test == 0:
                test = 1
                amount = int(
                    offer_tree.select_one(".list_level.imp.small")["rat"]
                )
        return amount


class Profile(AbstractWrapper):
    def info(self, resource):
        """Get profile"""
        if isinstance(resource, str) and resource in util.ITEM_KEYS:
            resource = util.ITEM_KEYS[resource]
        path = "storage/listed/{}".format(resource)
        response = self.middleware.get(path)
        soup = BeautifulSoup(response, "html.parser")
        offers_tree = soup.find_all(class_="list_link")
        test = 0
        profiel = 0
        for offer_tree in offers_tree:
            if test == 0:
                test = 1
                profiel = int(
                    re.sub(
                        r"^.*\/",
                        "",
                        offer_tree.select_one(".results_date")["action"],
                    )
                )
        return profiel


class BuyMarket(AbstractWrapper):
    def info(self, resource, price):
        current_price = CurrentPrice(authentication).info(resource) / 100
        if current_price < price:
            profiel = Profile(authentication).info(resource)
            amount = Amount(authentication).info(resource)
            item = util.ITEM_KEYS[resource]
            self.middleware.post(
                "storage/buy/{}/{}/{}/{}".format(
                    item, profiel, amount, current_price
                )
            )
        return True
